@@ -3,9 +3,9 @@ package net
 import (
 	"context"
 	"net"
-	"os"
 	"sync"
 
+	"github.com/lmorg/murex/lang/tty"
 	"github.com/lmorg/murex/utils"
 )
 
@@ -71,7 +71,7 @@ func (n *Net) Close() {
 	if n.dependents == 0 {
 		err := n.conn.Close()
 		if err != nil {
-			os.Stderr.WriteString(err.Error() + utils.NewLineString)
+			tty.Stderr.WriteString(err.Error() + utils.NewLineString)
 		}
 	}
 
