@@ -27,7 +27,7 @@ func (a *Aliases) Add(name string, alias []string) {
 // Exists checks if alias exists in table
 func (a *Aliases) Exists(name string) (exists bool) {
 	a.mutex.Lock()
-	exists = len(a.aliases[name]) > 0
+	exists = a.aliases[name] != nil
 	a.mutex.Unlock()
 	return exists
 }
@@ -43,12 +43,13 @@ func (a *Aliases) Get(name string) (alias []string) {
 // Delete an alias
 func (a *Aliases) Delete(name string) error {
 	a.mutex.Lock()
-	defer a.mutex.Unlock()
 
 	if len(a.aliases[name]) == 0 {
+		a.mutex.Unlock()
 		return errors.New("alias does not exist")
 	}
 	delete(a.aliases, name)
+	a.mutex.Unlock()
 	return nil
 }
 
