@@ -1,15 +1,18 @@
 package lang
 
 import (
+	"errors"
 	"fmt"
 	"os"
 	"strconv"
+	"strings"
 	"sync"
 	"time"
 
 	"github.com/lmorg/murex/builtins/pipes/streams"
 	"github.com/lmorg/murex/lang/ref"
 	"github.com/lmorg/murex/lang/types"
+	"github.com/lmorg/murex/utils/alter"
 	"github.com/lmorg/murex/utils/envvars"
 	"github.com/lmorg/murex/utils/json"
 )
@@ -26,9 +29,14 @@ func errVarNoParam(i int, err error) error {
 	return fmt.Errorf("variable '%d' cannot be defined: %s", i, err.Error())
 }
 
+var (
+	errZeroLengthPath = errors.New("zero length path")
+)
+
 // Reserved variable names. Set as constants so any typos of these names within
 // the code will be raised as compiler errors
 const (
+	DOT        = ""
 	SELF       = "SELF"
 	ARGS       = "ARGS"
 	PARAMS     = "PARAMS"
@@ -36,6 +44,8 @@ const (
 	MUREX_ARGS = "MUREX_ARGS"
 	HOSTNAME   = "HOSTNAME"
 	PWD        = "PWD"
+	ENV        = "ENV"
+	GLOBAL     = "GLOBAL"
 )
 
 // Variables is a table of all the variables. This will be local to the scope's
@@ -66,19 +76,47 @@ func NewGlobals() *Variables {
 
 // variable is an individual variable or global variable
 type variable struct {
-	DataType string
-	Value    interface{}
-	String   string
-	Modify   time.Time
-	FileRef  *ref.File // only needed for globals
+	DataType    string
+	Value       interface{}
+	String      string
+	IsInterface bool
+	Modify      time.Time
+	FileRef     *ref.File // only needed for globals
 }
 
 // GetValue return the value of a variable. If a variable does not exist then
 // GetValue will return nil. Please check if p.Config.Get("proc", "strict-vars", "bool")
 // matters for your usage of GetValue because this API doesn't care. If in doubt
 // use GetString instead.
-func (v *Variables) GetValue(name string) (interface{}, error) {
+func (v *Variables) GetValue(path string) (interface{}, error) {
+	if path == "." {
+		return v.getValue(DOT)
+	}
+
+	split := strings.Split(path, ".")
+	switch len(split) {
+	case 0:
+		return nil, errors.New("zero length path")
+	case 1:
+		return v.getValue(split[0])
+	default:
+		val, err := v.getValue(split[0])
+		if err != nil {
+			return nil, err
+		}
+
+		return ElementLookup(val, "."+strings.Join(split[1:], "."))
+	}
+}
+
+func (v *Variables) getValue(name string) (interface{}, error) {
 	switch name {
+	case ENV:
+		return getEnvVarValue(), nil
+
+	case GLOBAL:
+		return getGlobalValues(), nil
+
 	case SELF:
 		return getVarSelf(v.process), nil
 
@@ -89,13 +127,13 @@ func (v *Variables) GetValue(name string) (interface{}, error) {
 		return v.process.Scope.Parameters.StringArray(), nil
 
 	case MUREX_EXE:
-		return getVarMurexExe(), nil
+		return getVarMurexExeValue()
 
 	case HOSTNAME:
 		return getHostname(), nil
 
 	case PWD:
-		return getPwd(), nil
+		return getPwdValue()
 
 	case "0":
 		return v.process.Scope.Name.String(), nil
@@ -110,15 +148,15 @@ func (v *Variables) GetValue(name string) (interface{}, error) {
 	}
 
 	if v.global {
-		return v.getValue(name), nil
+		return v.getValueValue(name), nil
 	}
 
-	value := v.getValue(name)
+	value := v.getValueValue(name)
 	if value != nil {
 		return value, nil
 	}
 
-	value = GlobalVariables.getValue(name)
+	value = GlobalVariables.getValueValue(name)
 	if value != nil {
 		return value, nil
 	}
@@ -136,7 +174,7 @@ func (v *Variables) GetValue(name string) (interface{}, error) {
 	return nil, nil
 }
 
-func (v *Variables) getValue(name string) (value interface{}) {
+func (v *Variables) getValueValue(name string) interface{} {
 	v.mutex.Lock()
 	variable := v.vars[name]
 	if variable == nil {
@@ -144,35 +182,87 @@ func (v *Variables) getValue(name string) (value interface{}) {
 		return nil
 	}
 
-	value = variable.Value
-	v.mutex.Unlock()
+	if variable.IsInterface {
+		value := variable.Value.(MxInterface).GetValue()
 
+		v.mutex.Unlock()
+		return value
+	}
+
+	value := variable.Value
+
+	v.mutex.Unlock()
 	return value
 }
 
 // GetString returns a string representation of the data stored in the requested variable
-func (v *Variables) GetString(name string) (string, error) {
+func (v *Variables) GetString(path string) (string, error) {
+	if path == "." {
+		return v.getString(DOT)
+	}
+
+	split := strings.Split(path, ".")
+	switch len(split) {
+	case 0:
+		return "", errZeroLengthPath
+	case 1:
+		return v.getString(split[0])
+	default:
+		val, err := v.getValue(split[0])
+		if err != nil {
+			return "", err
+		}
+
+		val, err = ElementLookup(val, "."+strings.Join(split[1:], "."))
+		if err != nil {
+			return "", err
+		}
+
+		switch val.(type) {
+		case int, float64, string, bool, nil:
+			s, err := types.ConvertGoType(val, types.String)
+			if err != nil {
+				return "", err
+			}
+			return s.(string), nil
+		default:
+			dataType := v.GetDataType(split[0])
+			b, err := MarshalData(v.process, dataType, val)
+			return string(b), err
+		}
+	}
+}
+
+func (v *Variables) getString(name string) (string, error) {
 	switch name {
+	case ENV:
+		b, err := json.Marshal(getEnvVarValue(), v.process.Stdout.IsTTY())
+		return string(b), err
+
+	case GLOBAL:
+		b, err := json.Marshal(getGlobalValues(), v.process.Stdout.IsTTY())
+		return string(b), err
+
 	case SELF:
-		b, _ := json.Marshal(getVarSelf(v.process), v.process.Stdout.IsTTY())
-		return string(b), nil
+		b, err := json.Marshal(getVarSelf(v.process), v.process.Stdout.IsTTY())
+		return string(b), err
 
 	case ARGS:
-		b, _ := json.Marshal(getVarArgs(v.process), v.process.Stdout.IsTTY())
-		return string(b), nil
+		b, err := json.Marshal(getVarArgs(v.process), v.process.Stdout.IsTTY())
+		return string(b), err
 
 	case PARAMS:
-		b, _ := json.Marshal(v.process.Scope.Parameters.StringArray(), v.process.Stdout.IsTTY())
-		return string(b), nil
+		b, err := json.Marshal(v.process.Scope.Parameters.StringArray(), v.process.Stdout.IsTTY())
+		return string(b), err
 
 	case MUREX_EXE:
-		return getVarMurexExe().(string), nil
+		return os.Executable()
 
 	case HOSTNAME:
 		return getHostname(), nil
 
 	case PWD:
-		return getPwd(), nil
+		return os.Getwd()
 
 	case "0":
 		return v.process.Scope.Name.String(), nil
@@ -187,16 +277,16 @@ func (v *Variables) GetString(name string) (string, error) {
 	}
 
 	if v.global {
-		val, _ := v.getString(name)
+		val, _ := v.getStringValue(name)
 		return val, nil
 	}
 
-	s, exists := v.getString(name)
+	s, exists := v.getStringValue(name)
 	if exists {
 		return s, nil
 	}
 
-	s, exists = GlobalVariables.getString(name)
+	s, exists = GlobalVariables.getStringValue(name)
 	if exists {
 		return s, nil
 	}
@@ -212,7 +302,7 @@ func (v *Variables) GetString(name string) (string, error) {
 	return s, nil
 }
 
-func (v *Variables) getString(name string) (string, bool) {
+func (v *Variables) getStringValue(name string) (string, bool) {
 	v.mutex.Lock()
 	variable := v.vars[name]
 	if variable == nil {
@@ -220,14 +310,66 @@ func (v *Variables) getString(name string) (string, bool) {
 		return "", false
 	}
 
+	if variable.IsInterface {
+		s := variable.Value.(MxInterface).GetString()
+
+		v.mutex.Unlock()
+		return s, true
+	}
+
 	s := variable.String
 	v.mutex.Unlock()
 	return s, true
 }
 
 // GetDataType returns the data type of the variable stored in the referenced VarTable
-func (v *Variables) GetDataType(name string) string {
+func (v *Variables) GetDataType(path string) string {
+	if path == "." {
+		return v.getDataType(DOT)
+	}
+
+	split := strings.Split(path, ".")
+	switch len(split) {
+	case 0:
+		return ""
+	case 1:
+		return v.getDataType(split[0])
+	default:
+		val, err := v.getValue(split[0])
+		if err != nil {
+			return v.getDataType(split[0])
+		}
+
+		val, err = ElementLookup(val, "."+strings.Join(split[1:], "."))
+		if err != nil {
+			return v.getDataType(split[0])
+		}
+
+		switch val.(type) {
+		case int:
+			return types.Integer
+		case float64:
+			return types.Number
+		case string, []byte, []rune:
+			return types.String
+		case bool:
+			return types.Boolean
+		case nil:
+			return types.Null
+		default:
+			return v.getDataType(split[0])
+		}
+	}
+}
+
+func (v *Variables) getDataType(name string) string {
 	switch name {
+	case ENV:
+		return types.Json
+
+	case GLOBAL:
+		return types.Json
+
 	case SELF:
 		return types.Json
 
@@ -238,10 +380,10 @@ func (v *Variables) GetDataType(name string) string {
 		return types.Json
 
 	case MUREX_EXE:
-		return types.String
+		return types.Path
 
 	case PWD:
-		return types.String
+		return types.Path
 
 	case "0":
 		return types.String
@@ -255,16 +397,16 @@ func (v *Variables) GetDataType(name string) string {
 	}
 
 	if v.global {
-		dt, _ := v.getDataType(name)
+		dt, _ := v.getDataTypeValue(name)
 		return dt
 	}
 
-	s, exists := v.getDataType(name)
+	s, exists := v.getDataTypeValue(name)
 	if exists {
 		return s
 	}
 
-	s, exists = GlobalVariables.getDataType(name)
+	s, exists = GlobalVariables.getDataTypeValue(name)
 	if exists {
 		return s
 	}
@@ -278,7 +420,7 @@ func (v *Variables) GetDataType(name string) string {
 	return types.Null
 }
 
-func (v *Variables) getDataType(name string) (string, bool) {
+func (v *Variables) getDataTypeValue(name string) (string, bool) {
 	v.mutex.Lock()
 	variable := v.vars[name]
 	if variable == nil {
@@ -291,11 +433,53 @@ func (v *Variables) getDataType(name string) (string, bool) {
 	return dt, true
 }
 
+func errCannotUpdateNested(name string, err error) error {
+	return fmt.Errorf("cannot update element inside %s: %s", name, err.Error())
+}
+
+func (v *Variables) Set(p *Process, path string, value interface{}, dataType string) error {
+	split := strings.Split(path, ".")
+	switch len(split) {
+	case 0:
+		return errZeroLengthPath
+	case 1:
+		return v.set(p, split[0], value, dataType, nil)
+	default:
+		variable, err := v.getValue(split[0])
+		if err != nil {
+			return errCannotUpdateNested(split[0], err)
+		}
+
+		variable, err = alter.Alter(p.Context, variable, split[1:], value)
+		if err != nil {
+			return errCannotUpdateNested(split[0], err)
+		}
+		err = v.set(p, split[0], variable, v.getNestedDataType(split[0], dataType), split[1:])
+		if err != nil {
+			return errCannotUpdateNested(split[0], err)
+		}
+		return nil
+	}
+}
+
+func (v *Variables) getNestedDataType(name string, dataType string) string {
+	if name == GLOBAL {
+		return dataType
+	}
+	return v.GetDataType(name)
+}
+
 // Set writes a variable
-func (v *Variables) Set(p *Process, name string, value interface{}, dataType string) error {
+func (v *Variables) set(p *Process, name string, value interface{}, dataType string, changePath []string) error {
 	switch name {
 	case SELF, ARGS, PARAMS, MUREX_EXE, MUREX_ARGS, HOSTNAME, PWD, "_":
 		return errVariableReserved(name)
+	case ENV:
+		return setEnvVar(value, changePath)
+	case GLOBAL:
+		return setGlobalVar(p, value, changePath, dataType)
+	case DOT:
+		goto notReserved
 	}
 	for _, r := range name {
 		if r < '0' || r > '9' {
@@ -305,53 +489,59 @@ func (v *Variables) Set(p *Process, name string, value interface{}, dataType str
 	return errVariableReserved(name)
 
 notReserved:
-	var (
-		s     string
-		iface interface{}
-		err   error
-	)
-
-	/*if name == "TestBug507" {
-		panic(fmt.Sprintf("value of value (%s) == '%v' (%T, %s)", name, value, value, dataType))
-	}*/
-	switch v := value.(type) {
-	case float64, int, bool, nil:
-		s, err = varConvertPrimitive(value)
-		iface = value
-	case string:
-		s = v
-		if dataType != types.String && dataType != types.Generic {
-			iface, err = varConvertString([]byte(v), dataType)
-		} else {
-			iface = s
-		}
-	case []byte:
-		s = string(v)
-		if dataType != types.String && dataType != types.Generic {
-			iface, err = varConvertString(v, dataType)
-		} else {
-			iface = s
-		}
-	case []rune:
-		s = string(v)
-		if dataType != types.String && dataType != types.Generic {
-			iface, err = varConvertString([]byte(string(v)), dataType)
-		} else {
-			iface = s
-		}
-	default:
-		s, err = varConvertInterface(v, dataType)
-		iface = value
-	}
-	if err != nil {
-		return err
-	}
 
 	fileRef := v.process.FileRef
 	if v.global {
 		fileRef = p.FileRef
 	}
 
+	mxi := MxInterfaces[dataType]
+	if mxi != nil {
+		mxvar := v.vars[name]
+		if mxvar != nil && mxvar.IsInterface {
+
+			v.mutex.Lock()
+
+			err := mxvar.Value.(MxInterface).Set(value, changePath)
+			if err != nil {
+				v.vars[name].Modify = time.Now()
+			}
+
+			v.mutex.Unlock()
+
+			return err
+		}
+
+		s, _, err := convertDataType(p, value, dataType)
+		if err != nil {
+			return err
+		}
+
+		mxi, err := mxi.New(s)
+		if err != nil {
+			return err
+		}
+
+		v.mutex.Lock()
+
+		v.vars[name] = &variable{
+			Value:       mxi,
+			DataType:    dataType,
+			Modify:      time.Now(),
+			FileRef:     fileRef,
+			IsInterface: true,
+		}
+
+		v.mutex.Unlock()
+
+		return nil
+	}
+
+	s, iface, err := convertDataType(p, value, dataType)
+	if err != nil {
+		return err
+	}
+
 	v.mutex.Lock()
 
 	v.vars[name] = &variable{
@@ -367,8 +557,82 @@ notReserved:
 	return nil
 }
 
+func setGlobalVar(p *Process, v interface{}, changePath []string, dataType string) (err error) {
+	if len(changePath) == 0 {
+		return fmt.Errorf("invalid use of $%s. Expecting a global variable name, eg `$GLOBAL.example`", ENV)
+	}
+
+	switch t := v.(type) {
+	case map[string]interface{}:
+		return GlobalVariables.Set(ShellProcess, changePath[0], t[changePath[0]], dataType)
+
+	default:
+		return fmt.Errorf("expecting a map of global variables. Instead got a %T", t)
+	}
+}
+
+func setEnvVar(v interface{}, changePath []string) (err error) {
+	var value interface{}
+
+	if len(changePath) == 0 {
+		return fmt.Errorf("invalid use of $%s. Expecting an environmental variable name, eg `$ENV.EXAMPLE`", ENV)
+	}
+
+	switch t := v.(type) {
+	case map[string]interface{}:
+		value, err = types.ConvertGoType(t[changePath[0]], types.String)
+		if err != nil {
+			return err
+		}
+
+	default:
+		return fmt.Errorf("expecting a map of environmental variables. Instead got a %T", t)
+	}
+
+	return os.Setenv(changePath[0], value.(string))
+}
+
 const errCannotStoreVariable = "cannot store variable"
 
+func convertDataType(p *Process, value interface{}, dataType string) (string, interface{}, error) {
+	var (
+		s     string
+		iface interface{}
+		err   error
+	)
+
+	switch v := value.(type) {
+	case float64, int, bool, nil:
+		s, err = varConvertPrimitive(value)
+		iface = value
+	case string:
+		s = v
+		if dataType != types.String && dataType != types.Generic {
+			iface, err = varConvertString(p, []byte(v), dataType)
+		} else {
+			iface = s
+		}
+	case []byte:
+		s = string(v)
+		if dataType != types.String && dataType != types.Generic {
+			iface, err = varConvertString(p, v, dataType)
+		} else {
+			iface = s
+		}
+	case []rune:
+		s = string(v)
+		if dataType != types.String && dataType != types.Generic {
+			iface, err = varConvertString(p, []byte(string(v)), dataType)
+		} else {
+			iface = s
+		}
+	default:
+		s, err = varConvertInterface(v, dataType)
+		iface = value
+	}
+	return s, iface, err
+}
+
 func varConvertPrimitive(value interface{}) (string, error) {
 	s, err := types.ConvertGoType(value, types.String)
 	if err != nil {
@@ -377,7 +641,7 @@ func varConvertPrimitive(value interface{}) (string, error) {
 	return s.(string), nil
 }
 
-func varConvertString(value []byte, dataType string) (interface{}, error) {
+func varConvertString(parent *Process, value []byte, dataType string) (interface{}, error) {
 	UnmarshalData := Unmarshallers[dataType]
 
 	// no unmarshaller exists so lets just return the bare string
@@ -386,6 +650,7 @@ func varConvertString(value []byte, dataType string) (interface{}, error) {
 	}
 
 	p := new(Process)
+	p.Config = parent.Config
 	p.Stdin = streams.NewStdin()
 	_, err := p.Stdin.Write([]byte(value))
 	if err != nil {
@@ -470,5 +735,7 @@ func DumpVariables(p *Process) map[string]interface{} {
 	m[MUREX_ARGS], _ = p.Variables.GetValue(MUREX_ARGS)
 	m[HOSTNAME], _ = p.Variables.GetValue(HOSTNAME)
 	m[PWD], _ = p.Variables.GetValue(PWD)
+	m[ENV], _ = p.Variables.GetValue(ENV)
+	m[GLOBAL] = nil
 	return m
 }
