@@ -1,3 +1,4 @@
+//go:build !windows && !plan9 && !js
 // +build !windows,!plan9,!js
 
 package readline
@@ -23,9 +24,9 @@ func (rl *Instance) launchEditor(multiline []rune) ([]rune, error) {
 
 	cmd := exec.Command(editor, name)
 
-	cmd.Stdin = os.Stdin
-	cmd.Stdout = os.Stdout
-	cmd.Stderr = os.Stderr
+	cmd.Stdin = replica
+	cmd.Stdout = primary
+	cmd.Stderr = primary
 
 	if err := cmd.Start(); err != nil {
 		return multiline, err
