@@ -36,10 +36,13 @@ func (rl *Instance) getTabCompletion() {
 	rl.delayedTabContext = DelayedTabContext{rl: rl}
 	rl.delayedTabContext.Context, rl.delayedTabContext.cancel = context.WithCancel(context.Background())
 
-	prefix, suggestions, descriptions, displayType := rl.TabCompleter(rl.line, rl.pos, rl.delayedTabContext)
+	rl.tcr = rl.TabCompleter(rl.line.Runes(), rl.line.RunePos(), rl.delayedTabContext)
+	if rl.tcr == nil {
+		return
+	}
 
 	rl.tabMutex.Lock()
-	rl.tcPrefix, rl.tcSuggestions, rl.tcDescriptions, rl.tcDisplayType = prefix, suggestions, descriptions, displayType
+	rl.tcPrefix, rl.tcSuggestions, rl.tcDescriptions, rl.tcDisplayType = rl.tcr.Prefix, rl.tcr.Suggestions, rl.tcr.Descriptions, rl.tcr.DisplayType
 	if len(rl.tcDescriptions) == 0 {
 		// probably not needed, but just in case someone doesn't initialize the
 		// map in their API call.
@@ -72,8 +75,8 @@ func (rl *Instance) writeTabCompletion(resetCursorPos bool) {
 		return
 	}
 
-	_, posY := lineWrapPos(rl.promptLen, rl.pos, rl.termWidth)
-	_, lineY := lineWrapPos(rl.promptLen, len(rl.line), rl.termWidth)
+	_, posY := rl.lineWrapCellPos()
+	_, lineY := rl.lineWrapCellLen()
 	moveCursorDown(rl.hintY + lineY - posY)
 	print("\r\n" + seqClearScreenBelow)
 
