@@ -1,6 +1,6 @@
 package readline
 
-import "regexp"
+import "strings"
 
 func (rl *Instance) backspaceTabFind() {
 	if len(rl.tfLine) > 0 {
@@ -11,7 +11,6 @@ func (rl *Instance) backspaceTabFind() {
 
 func (rl *Instance) updateTabFind(r []rune) {
 	rl.tfLine = append(rl.tfLine, r...)
-	rl.hintText = append([]rune("regex find: "), rl.tfLine...)
 
 	defer func() {
 		rl.clearHelpers()
@@ -23,22 +22,30 @@ func (rl *Instance) updateTabFind(r []rune) {
 	defer rl.tabMutex.Unlock()
 
 	if len(rl.tfLine) == 0 {
+		rl.hintText = rFindSearchPart
 		rl.tfSuggestions = append(rl.tcSuggestions, []string{}...)
 		return
 	}
 
-	rx, err := regexp.Compile("(?i)" + string(rl.tfLine))
+	var (
+		find findT
+		err  error
+	)
+
+	find, rl.rFindSearch, rl.rFindCancel, err = newFuzzyFind(string(rl.tfLine))
 	if err != nil {
 		rl.tfSuggestions = []string{err.Error()}
 		return
 	}
 
+	rl.hintText = append(rl.rFindSearch, rl.tfLine...)
+
 	rl.tfSuggestions = make([]string, 0)
 	for i := range rl.tcSuggestions {
-		if rx.MatchString(rl.tcSuggestions[i]) {
+		if find.MatchString(strings.TrimSpace(rl.tcSuggestions[i])) {
 			rl.tfSuggestions = append(rl.tfSuggestions, rl.tcSuggestions[i])
 
-		} else if rl.tcDisplayType == TabDisplayList && rx.MatchString(rl.tcDescriptions[rl.tcSuggestions[i]]) {
+		} else if rl.tcDisplayType == TabDisplayList && find.MatchString(rl.tcDescriptions[rl.tcSuggestions[i]]) {
 			// this is a list so lets also check the descriptions
 			rl.tfSuggestions = append(rl.tfSuggestions, rl.tcSuggestions[i])
 		}
@@ -51,7 +58,7 @@ func (rl *Instance) resetTabFind() {
 	if rl.modeAutoFind {
 		rl.hintText = []rune{}
 	} else {
-		rl.hintText = []rune("Cancelled regex suggestion find.")
+		rl.hintText = rl.rFindCancel
 	}
 
 	rl.clearHelpers()
